#!/usr/bin/perl -w
###############################################################
# Simple Redirection Script                   Version: v1.01  #
###############################################################
# Written By:   Mark R Hives                                  #
# Filename:     m_redir.pl                                    #
# Release Date:  8-Apr-2004  [v1.01]                          #
#               23-Mar-2003  [v1.00]                          #
#                                                             #
# Description:                                                #
#  A simple redirection script with counters.                 #
#                                                             #
# How to use this script:                                     #
#  To install create a directory on your web server to        #
#  contain the support files (usually called 'm_redir_data'). #
#  Next create an empty file called 'lock-file' within it     #
#  and change the '$sd' variable, in this script, to contain  #
#  the path to the new directory. Finally upload the          #
#  modified script to your cgi-bin.                           #
#                                                             #
#  Please note for improved security you may wish to change   #
#  the '$reset_code', '$show_code' and '$down_code' variables #
#  below.                                                     #
#                                                             #
#  To use the script in a hyperlink:                          #
#   <a href="http://www.your-domain.com/cgi-bin/m_redir.pl    #
#   ?http://www.other-domain.com/pagename.html">Link Text</a> #
#                                                             #
#  To view the current state of the counters type the         #
#  following into your web browser's address bar (note. if    #
#  you have changed the value of the $show_code variable      #
#  then you should replace the ".show" with your new value):  #
#   http://www.your-domain.com/cgi-bin/m_redir.pl?.show       #
#                                                             #
#  To reset all the counters type the following into your     #
#  web browser's address bar (note. if you have changed the   #
#  value of the $reset_code variable then you should replace  #
#  the ".reset" with your new value):                         #
#   http://www.your-domain.com/cgi-bin/m_redir.pl?.reset      #
#                                                             #
#                                                             #
###############################################################

my (%ud, %counters, $sd, $reset_code, $show_code, @tl, $x);

###############################################################
# CONFIGURATION SECTION                                       #
###############################################################

#Path to Support Directory
$sd          = "./m_redir_data";

#Vote system Reset Code
$reset_code  = ".reset";

#Vote system Show Code
$show_code   = ".show";

#Vote system Download Code
$down_code   = ".download";

###############################################################


#Get and parse Url data
%ud = ();
read_url_data();


#Hashes and Locks - OPEN
open(LOCK_FILE, "+< $sd/lock-file") or die_msg("OPEN LOCKFILE - Failed");
flock(LOCK_FILE, 2) or die_msg("FLOCK 2 - Failed");
dbmopen(%counters, "$sd/counters", 0666) or die_msg("DBMOPEN counters - Failed");


##
if ($ud{'clean'} eq $reset_code) {
   %counters = ();
   die_msg("<p>Counters have been Reset.</p>");

} elsif ($ud{'clean'} eq $show_code) {
   output_header();
   show_results();
   output_footer();

} elsif ($ud{'clean'} eq $down_code) {
   download_as_file();

} elsif ($ud{'status'} eq 'valid') {
   $counters{'url_'    . $ud{'clean'}}  += 1;
   $counters{'domain_' . $ud{'domain'}} += 1;
   print "location: ", $ud{'clean'}, "\n\n";

} else {
   die_msg("ELSE ERROR ?");
}


#Hashes and Locks - CLOSE
dbmclose(%counters) or die_msg("DBMCLOSE counters - Failed");
flock(LOCK_FILE, 8) or die_msg("FLOCK 8 - Failed");
close LOCK_FILE;

############################################################################


sub show_results {
   my($x, @keylist, $total);
   @keylist = keys %counters;
   $total = 0;
   foreach $x (@keylist) {
      $total = $total + $counters{$x};
   }
   $total = $total / 2;

   print "<table border=0 width=600>\n",
         "<tr><td colspan=3 align=center><a href=\"m_redir.pl?$down_code\">Click here to Download a plain text version</a></td></tr>\n",
         "<tr><td colspan=3><font size=+1><b>Listed by URL</b></font></td></tr>\n";

   foreach $x (sort @keylist) {
      if(index($x, "url_") == 0) {
         print "<tr><td><a href=\"", substr($x, 4, 9999),
               "\">", substr($x, 4, 9999), "</a></td>\n",
               "    <td>", $counters{$x}, "</td>\n",
               "    <td align=right nowrap>";
         printf("%.1f %%</td>\n", ($counters{$x} * 100 / $total));
      }
   }

   print "<tr><td colspan=3>&nbsp;</td></tr>\n",
         "<tr><td colspan=3><font size=+1><b>Listed by Domain</b></font></td></tr>\n";

   foreach $x (sort @keylist) {
      if(index($x, "domain_") == 0) {
         print "<tr><td><a href=\"http://", substr($x, 7, 9999),
               "/\">", substr($x, 7, 9999), "</a></td>\n",
               "    <td>", $counters{$x}, "</td>\n",
               "    <td align=right nowrap>";
         printf("%.1f %%</td>\n", ($counters{$x} * 100 / $total));
      }
   }
   print "</table>\n<br>\n";
}

sub download_as_file {
   my($x, @keylist, $total);
   @keylist = keys %counters;
   $total = 0;
   foreach $x (@keylist) {
      $total = $total + $counters{$x};
   }
   $total = $total / 2;

   print "Content-Type: text/plain; name=\"stats.txt\"\n",
         "Content-Disposition: attachment; filename=\"stats.txt\"\n", 
         "Pragma: no-cache\n",
         "Content-Transfer-Encoding: 7bit\n\n",
         "Listed by URL:\015\012";

   foreach $x (sort @keylist) {
      if(index($x, "url_") == 0) {
         print substr($x, 4, 9999), "\t", $counters{$x}, "\t";
         printf("%.1f %%\015\012", ($counters{$x} * 100 / $total));
      }
   }

   print "\015\012Listed by Domain:\015\012";

   foreach $x (sort @keylist) {
      if(index($x, "domain_") == 0) {
         print substr($x, 7, 9999), "\t", $counters{$x}, "\t";
         printf("%.1f %%\015\012", ($counters{$x} * 100 / $total));
      }
   }
   print "\015\012EOF";
}


sub die_msg    {
   output_header();
   print @_;
   #print %ENV;
   output_footer();
   exit;
}

sub read_url_data {
   my ($buffer);
   $buffer = ($ENV{'QUERY_STRING'} || "") . "";
   $buffer =~ tr/+/ /;
   $buffer =~ s/%([\da-f]{2})/chr(hex($1))/egi;

   if ($buffer =~ m|^http://([^/:]+):?[0-9]*/|i) {
      $ud{'status'} = "valid";
      $ud{'clean'}  = $buffer;
      $ud{'domain'} = $1;
   } else {
      $ud{'status'} = "invalid";
      $ud{'clean'}  = $buffer;
      $ud{'domain'} = "";
   }
}


sub output_header {
   print "content-type: text/html\n\n";
   print "<html>\n<head>\n<title>Simple Redirection Script v1.01</title>\n</head>\n",
         "<body>\n";
   print "<table width=100% border=0><tr><td>\n",
         "<font size=5 color=#333333> Simple Redirection Script v1.01 </font>",
         "<br> &nbsp; ",
         "</td></tr></table>\n";
}

sub output_footer {
   print "<table width=100% border=0 cellpadding=0 cellspacing=0>\n",
         "<tr><td align=left width=50%>\n",
         "<font size=1 color=#999999>Simple Redirection Script v1.01</font>\n",
         "</td><td align=right width=50%>",
         "<font size=1 color=#999999>&copy; 2003 Mark R Hives</font>\n",
         "</td></tr></table>\n";

   print "</body>\n</html>";
}


